from abaqus import *
from abaqusConstants import *
from caeModules import *
from driverUtils import *


def test_mdb():
    executeOnCaeStartup()

    # Model
    model = mdb.models["Model-1"]

    # Part
    sketch = model.ConstrainedSketch(name="sketch", sheetSize=1.0)
    sketch.rectangle((0, 0), (1, 1))
    part = model.Part(name="part", dimensionality=THREE_D, type=DEFORMABLE_BODY)
    part.BaseSolidExtrude(sketch=sketch, depth=1)

    # Create sets
    part.Set(name="set-all", cells=part.cells.findAt(coordinates=((0.5, 0.5, 0.5),)))
    part.Set(name="set-bottom", faces=part.faces.findAt(coordinates=((0.5, 0.5, 0.0),)))
    part.Set(name="set-top", faces=part.faces.findAt(coordinates=((0.5, 0.5, 1.0),)))
    part.Surface(name="surface-top", side1Faces=part.faces.findAt(coordinates=((0.5, 0.5, 1.0),)))

    # Assembly
    model.rootAssembly.DatumCsysByDefault(CARTESIAN)
    model.rootAssembly.Instance(name="instance", part=part, dependent=ON)

    # Material
    material = model.Material(name="material")
    material.Elastic(table=((1000, 0.2),))
    material.Density(table=((2500,),))

    # Section
    model.HomogeneousSolidSection(name="section", material="material", thickness=None)
    part.SectionAssignment(region=part.sets["set-all"], sectionName="section")

    # Step
    step = model.StaticStep(
        name="Step-1",
        previous="Initial",
        description="",
        timePeriod=1.0,
        timeIncrementationMethod=AUTOMATIC,
        maxNumInc=100,
        initialInc=0.01,
        minInc=0.001,
        maxInc=0.1,
    )

    # Output request
    field = model.FieldOutputRequest("F-Output-1", createStepName="Step-1", variables=("S", "E", "U"))

    # Boundary condition
    bottom_instance = model.rootAssembly.instances["instance"].sets["set-bottom"]
    bc = model.DisplacementBC(name="BC-1", createStepName="Initial", region=bottom_instance, u3=SET)

    # Load
    top_instance = model.rootAssembly.instances["instance"].surfaces["surface-top"]
    pressure = model.Pressure("pressure", createStepName="Step-1", region=top_instance, magnitude=100)

    # Mesh
    elem1 = mesh.ElemType(elemCode=C3D8R)
    elem2 = mesh.ElemType(elemCode=C3D6)
    elem3 = mesh.ElemType(elemCode=C3D4)
    part.setElementType(regions=(part.cells,), elemTypes=(elem1, elem2, elem3))
    part.seedPart(size=0.1)
    part.generateMesh()

    # Job
    job = mdb.Job(name="Job-1", model="Model-1")
    job.writeInput()

    # Submit the job
    job.submit()
    job.waitForCompletion()

    # Save abaqus model
    # mdb.saveAs('compression.cae')



from __future__ import annotations

from typing_extensions import Literal

from abqpy.decorators import abaqus_class_doc, abaqus_method_doc

from ..Assembly.PartInstanceArray import PartInstanceArray
from ..UtilityAndView.abaqusConstants import (
    LAST_STEP,
    OFF,
    STEP_END,
    Boolean,
    SymbolicConstant,
)
from ..UtilityAndView.abaqusConstants import abaqusConstants as C
from .PredefinedField import PredefinedField


@abaqus_class_doc
class InitialState(PredefinedField):
    """The InitialState object stores the data for an initial state predefined field. The InitialState object is
    derived from the PredefinedField object.

    .. note::
        This object can be accessed by::

            import load
            mdb.models[name].predefinedFields[name]

        The corresponding analysis keywords are:

        - INSTANCE
    """

    #: A String specifying the repository key.
    name: str

    #: A PartInstanceArray object specifying the instances to which the predefined field is
    #: applied.
    instances: PartInstanceArray

    #: A String specifying the name of the job that generated the initial state data.
    fileName: str

    #: The SymbolicConstant LAST_STEP or an Int specifying the step from which the initial
    #: state values are to be read or the SymbolicConstant LAST_STEP. The default value is
    #: LAST_STEP.
    endStep: SymbolicConstant = LAST_STEP

    #: The SymbolicConstant STEP_END or an Int specifying the increment, interval or iteration
    #: of the step set in **endStep** or the SymbolicConstant STEP_END. The default value is
    #: STEP_END.
    endIncrement: SymbolicConstant = STEP_END

    #: A Boolean specifying whether to update the reference configuration based on the import
    #: data. The default value is OFF.
    updateReferenceConfiguration: Boolean = OFF

    @abaqus_method_doc
    def __init__(
        self,
        name: str,
        instances: PartInstanceArray,
        fileName: str,
        endStep: Literal[C.LAST_STEP] = LAST_STEP,
        endIncrement: Literal[C.STEP_END] = STEP_END,
        updateReferenceConfiguration: Boolean = OFF,
    ):
        """This method creates an InitialState predefined field object.

        .. note::
            This function can be accessed by::

                mdb.models[name].InitialState

        Parameters
        ----------
        name
            A String specifying the repository key.
        instances
            A PartInstanceArray object specifying the instances to which the predefined field is
            applied.
        fileName
            A String specifying the name of the job that generated the initial state data.
        endStep
            The SymbolicConstant LAST_STEP or an Int specifying the step from which the initial
            state values are to be read or the SymbolicConstant LAST_STEP. The default value is
            LAST_STEP.
        endIncrement
            The SymbolicConstant STEP_END or an Int specifying the increment, interval or iteration
            of the step set in **endStep** or the SymbolicConstant STEP_END. The default value is
            STEP_END.
        updateReferenceConfiguration
            A Boolean specifying whether to update the reference configuration based on the import
            data. The default value is OFF.

        Returns
        -------
        InitialState
            An InitialState object.
        """
        super().__init__()

    @abaqus_method_doc
    def setValues(
        self,
        endStep: Literal[C.LAST_STEP] = LAST_STEP,
        endIncrement: Literal[C.STEP_END] = STEP_END,
        updateReferenceConfiguration: Boolean = OFF,
    ):
        """This method modifies the InitialState object.

        Parameters
        ----------
        endStep
            The SymbolicConstant LAST_STEP or an Int specifying the step from which the initial
            state values are to be read or the SymbolicConstant LAST_STEP. The default value is
            LAST_STEP.
        endIncrement
            The SymbolicConstant STEP_END or an Int specifying the increment, interval or iteration
            of the step set in **endStep** or the SymbolicConstant STEP_END. The default value is
            STEP_END.
        updateReferenceConfiguration
            A Boolean specifying whether to update the reference configuration based on the import
            data. The default value is OFF.
        """
        ...



from __future__ import annotations

from abaqus.AbaqusCAEDisplayPreferences.caePrefsAccess import (
    getDisplayNamesInGuiPreferences,
    getGuiPrefsFileName,
    openGuiPreferences,
    openSessionOptions,
    printValuesList,
)

__all__ = [
    "getGuiPrefsFileName",
    "getDisplayNamesInGuiPreferences",
    "printValuesList",
    "openGuiPreferences",
    "openSessionOptions",
]



from __future__ import annotations

from abqpy.decorators import abaqus_class_doc, abaqus_method_doc


@abaqus_class_doc
class HeatGeneration:
    """The HeatGeneration object includes volumetric heat generation in heat transfer analyses.

    .. note::
        This object can be accessed by::

            import material
            mdb.models[name].materials[name].heatGeneration
            import odbMaterial
            session.odbs[name].materials[name].heatGeneration

        The corresponding analysis keywords are:

        - HEAT GENERATION
    """

    @abaqus_method_doc
    def __init__(self):
        """This method creates a HeatGeneration object.

        .. note::
            This function can be accessed by::

                mdb.models[name].materials[name].HeatGeneration
                session.odbs[name].materials[name].HeatGeneration

        Returns
        -------
        HeatGeneration
            A HeatGeneration object.
        """
        ...



from __future__ import annotations

from abqpy.decorators import abaqus_class_doc, abaqus_method_doc

from ..UtilityAndView.abaqusConstants import OFF, Boolean


@abaqus_class_doc
class Crack:
    """The Crack object is the abstract base type for ContourIntegral and future crack objects.

    .. note::
        This object can be accessed by::

            import part
            mdb.models[name].parts[name].engineeringFeatures.cracks[name]
            import assembly
            mdb.models[name].rootAssembly.engineeringFeatures.cracks[name]
    """

    #: A String specifying the repository key.
    name: str = ""

    #: A Boolean specifying whether the crack is suppressed or not. The default value is OFF.
    suppressed: Boolean = OFF

    @abaqus_method_doc
    def resume(self):
        """This method resumes the crack that was previously suppressed."""
        ...

    @abaqus_method_doc
    def suppress(self):
        """This method suppresses the crack."""
        ...



from __future__ import annotations

from abqpy.decorators import abaqus_class_doc, abaqus_method_doc

from ..UtilityAndView.abaqusConstants import DEFAULT_MODEL, SymbolicConstant
from .Leaf import Leaf


@abaqus_class_doc
class LeafFromModelNodeLabels(Leaf):
    """The LeafFromModelNodeLabels object can be used whenever a Leaf object is expected as an argument. Leaf
    objects are used to specify the items in a display group. Leaf objects are constructed as temporary objects,
    which are then used as arguments to DisplayGroup commands. The LeafFromModelNodeLabels object is derived
    from the Leaf object.

    .. note::
        This object can be accessed by::

            import displayGroupOdbToolset
    """

    #: A SymbolicConstant specifying the leaf type. Possible values are EMPTY_LEAF,
    #: DEFAULT_MODEL, ALL_ELEMENTS, ALL_NODES, and ALL_SURFACES.
    leafType: SymbolicConstant

    #: A sequence of Strings specifying expressions that denote node labels per part instance
    #: in the model. Each part instance node expression is a sequence of a String specifying
    #: the part instance name and a sequence of node expressions; for example,
    #: `(('partInstance1',(1,'7','3:15;3'),), ('partInstance2','8'),))`. The node expressions
    #: can be any of the following:An Int specifying a single node label; for example, `1`.A
    #: String specifying a single node label; for example, `'7'`.A String specifying a sequence
    #: of node labels; for example, `'3:5'` and `'3:15:3'`.
    nodeLabels: tuple[str, ...] = ()

    @abaqus_method_doc
    def __init__(self, nodeLabels: tuple):
        """This method creates a Leaf object from a sequence of node labels spanning several part instances.

        .. note::
            This function can be accessed by::

                LeafFromModelNodeLabels

        Parameters
        ----------
        nodeLabels
            A sequence of Strings specifying expressions that denote node labels per part instance
            in the model. Each part instance node expression is a sequence of a String specifying
            the part instance name and a sequence of node expressions; for example,
            `(('partInstance1',(1,'7','3:15;3'),), ('partInstance2','8'),))`. The node expressions
            can be any of the following:An Int specifying a single node label; for example, `1`.A
            String specifying a single node label; for example, `'7'`.A String specifying a sequence
            of node labels; for example, `'3:5'` and `'3:15:3'`.

        Returns
        -------
        LeafFromModelNodeLabels
            A LeafFromModelNodeLabels object.
        """
        super().__init__(DEFAULT_MODEL)



from __future__ import annotations

from abaqus.DisplayGroup.Leaf import Leaf
from abaqus.DisplayGroup.LeafFromDisplayGroup import LeafFromDisplayGroup
from abaqus.DisplayGroup.LeafFromElementLabels import LeafFromElementLabels
from abaqus.DisplayGroup.LeafFromElementSets import LeafFromElementSets
from abaqus.DisplayGroup.LeafFromElementVarRange import LeafFromElementVarRange
from abaqus.DisplayGroup.LeafFromModelElemLabels import LeafFromModelElemLabels
from abaqus.DisplayGroup.LeafFromModelNodeLabels import LeafFromModelNodeLabels
from abaqus.DisplayGroup.LeafFromNodeLabels import LeafFromNodeLabels
from abaqus.DisplayGroup.LeafFromNodeSets import LeafFromNodeSets
from abaqus.DisplayGroup.LeafFromNodeVarRange import LeafFromNodeVarRange
from abaqus.DisplayGroup.LeafFromOdbEdgePick import LeafFromOdbEdgePick
from abaqus.DisplayGroup.LeafFromOdbElementLayups import LeafFromOdbElementLayups
from abaqus.DisplayGroup.LeafFromOdbElementMaterials import LeafFromOdbElementMaterials
from abaqus.DisplayGroup.LeafFromOdbElementPick import LeafFromOdbElementPick
from abaqus.DisplayGroup.LeafFromOdbElementPlies import LeafFromOdbElementPlies
from abaqus.DisplayGroup.LeafFromOdbElementSections import LeafFromOdbElementSections
from abaqus.DisplayGroup.LeafFromOdbElementTypes import LeafFromOdbElementTypes
from abaqus.DisplayGroup.LeafFromOdbNodePick import LeafFromOdbNodePick
from abaqus.DisplayGroup.LeafFromPartInstance import LeafFromPartInstance
from abaqus.DisplayGroup.LeafFromSurfaceSets import LeafFromSurfaceSets
from abaqus.DisplayGroup.LeafFromSurfaceVarRange import LeafFromSurfaceVarRange
from abaqus.UtilityAndView.abaqusConstants import (
    ADD,
    ALL,
    ALL_DATUMS,
    ALL_DISTRIB_CONTINUUM_COUPLING,
    ALL_DISTRIB_COUPLING,
    ALL_DISTRIB_STRUCTURAL_COUPLING,
    ALL_ELEMENTS,
    ALL_GEOMETRY,
    ALL_KINEM_COUPLING,
    ALL_MPCS,
    ALL_NODES,
    ALL_RIGID_BODY,
    ALL_SHELL_SOLID_COUPLING,
    ALL_SURFACES,
    ALL_TIES,
    ASSEMBLY,
    DEFAULT_MODEL,
    EITHER,
    EMPTY_LEAF,
    INTERSECTION,
    NEGATE,
    REMOVE,
    REPLACE,
)

# Inspected from Abaqus 2021
__all__ = [
    "ADD",
    "ALL",
    "ALL_DATUMS",
    "ALL_DISTRIB_CONTINUUM_COUPLING",
    "ALL_DISTRIB_COUPLING",
    "ALL_DISTRIB_STRUCTURAL_COUPLING",
    "ALL_ELEMENTS",
    "ALL_GEOMETRY",
    "ALL_KINEM_COUPLING",
    "ALL_MPCS",
    "ALL_NODES",
    "ALL_RIGID_BODY",
    "ALL_SHELL_SOLID_COUPLING",
    "ALL_SURFACES",
    "ALL_TIES",
    "ASSEMBLY",
    # "ConstraintLeafType",
    # "ConstraintNodeLeafType",
    # "CsysLeafType",
    "DEFAULT_MODEL",
    # "DGCommonOptionsType",
    # "DGContourOptionsType",
    # "DGOrientationOptionsType",
    # "DGSuperimposeOptionsType",
    # "DGSymbolOptionsType",
    # "DisplayGroupKnotType",
    # "DisplayGroupLeafType",
    # "DisplayGroupOdbInstanceType",
    # "DisplayGroupType",
    "EITHER",
    "EMPTY_LEAF",
    # "EdgePickLeafType",
    # "ElementLayupLeafType",
    # "ElementMaterialLeafType",
    # "ElementPickLeafType",
    # "ElementPlyLeafType",
    # "ElementSectionLeafType",
    # "ElementTypeLeafType",
    # "ElementVariableRangeLeafType",
    # "EnumerationLeafType",
    "INTERSECTION",
    "Leaf",
    # "LeafFromConstraintNodes",
    "LeafFromDisplayGroup",
    "LeafFromElementLabels",
    "LeafFromElementSets",
    "LeafFromElementVarRange",
    "LeafFromModelElemLabels",
    "LeafFromModelNodeLabels",
    "LeafFromNodeLabels",
    "LeafFromNodeSets",
    "LeafFromNodeVarRange",
    # "LeafFromOdbCoordSystem",
    "LeafFromOdbEdgePick",
    "LeafFromOdbElementLayups",
    "LeafFromOdbElementMaterials",
    "LeafFromOdbElementPick",
    "LeafFromOdbElementPlies",
    "LeafFromOdbElementSections",
    "LeafFromOdbElementTypes",
    "LeafFromOdbNodePick",
    # "LeafFromOdbSurfacePick",
    "LeafFromPartInstance",
    # "LeafFromPath",
    "LeafFromSurfaceSets",
    "LeafFromSurfaceVarRange",
    # "LeafFromUserCoordSystem",
    # "LeafType",
    # "ModelElementLabelLeafType",
    # "ModelNodeLabelLeafType",
    "NEGATE",
    # "NodePickLeafType",
    # "NodeVarRangeLeafType",
    # "ODB",
    # "OdbDisplayOptionsType",
    # "PART",
    # "PART_ASSEMBLY",
    # "PartElementLabelLeafType",
    # "PartElementSetLeafType",
    # "PartInstanceLeafType",
    # "PartNodeLabelLeafType",
    # "PartNodeSetLeafType",
    # "PartSurfaceSetLeafType",
    # "PathLeafType",
    "REMOVE",
    "REPLACE",
    # "SurfVariableRangeLeafType",
    # "SurfacePickLeafType",
    "__builtins__",
    "__doc__",
    "__name__",
    "__package__",
]



from __future__ import annotations

from abqpy.decorators import abaqus_class_doc, abaqus_method_doc


@abaqus_class_doc
class CommandRegister:
    """This class allows you to derive a general class that can be queried from the GUI and is capable of
    notifying the GUI when the contents of the class change.

    .. note::
        This object can be accessed by::

            import customKernel
    """

    @abaqus_method_doc
    def __init__(self):
        """This class allows you to derive a general class that can be queried from the GUI and is capable of
        notifying the GUI when the contents of the class change.

        .. note::
            This function can be accessed by::

                customKernel.CommandRegister

        Returns
        -------
        CommandRegister
            A CommandRegister object.
        """
        ...



from __future__ import annotations

from .FXComposite import FXComposite
from .FXSrollArea import FXSrollArea


class FXScrollWindow(FXSrollArea):
    """The scroll window widget scrolls an arbitrary child window.

    Use the scroll window when parts of the user interface itself need to be scrolled, for example when
    applications need to run on small screens.
    """

    def __init__(self, p: FXComposite, opts: int = 0, x: int = 0, y: int = 0, w: int = 0, h: int = 0):
        """Construct a scroll window.

        Parameters
        ----------
        p : FXComposite

        opts : int

        x : int

        y : int

        w : int

        h : int
        """

    def contentWindow(self):
        """Return a pointer to the contents window."""

    def create(self):
        """Create server-side resources.

        Reimplemented from FXComposite.
        """

    def getContentHeight(self):
        """Return the height of the contents.

        Reimplemented from FXScrollArea. Reimplemented in AFXOptionTreeList.
        """

    def getContentWidth(self):
        """Return the width of the contents.

        Reimplemented from FXScrollArea. Reimplemented in AFXOptionTreeList.
        """

    def moveContents(self, x: int, y: int):
        """Move contents to the specified position. Reimplemented from FXScrollArea. Reimplemented in
        AFXOptionTreeList.

        Parameters
        ----------
        x : int

        y : int
        """



from __future__ import annotations

from typing_extensions import Literal

from abqpy.decorators import abaqus_class_doc, abaqus_method_doc

from ..UtilityAndView.abaqusConstants import GOURAUD, INFINITE, SymbolicConstant
from ..UtilityAndView.abaqusConstants import abaqusConstants as C
from .LightArray import LightArray


@abaqus_class_doc
class LightOptions:
    """The LightOptions object stores settings that control how objects are lit when the.

    **renderStyle** is set to SHADED. LightOptions objects are accessed in one of two ways:
    - The default light options. These settings are used as defaults when you start a
    session and by the Defaults button on the Light ConstrainedSketchOptions dialog box.
    - The light options associated with a particular viewport.
    The LightOptions object has no constructor; Abaqus creates the **defaultLightOptions**
    member when a session is started. When a new viewport is created, the **lightOptions**
    member is copied from the current viewport.

    .. note::
        This object can be accessed by::

            session.defaultLightOptions
            session.viewports[name].lightOptions
    """

    #: A SymbolicConstant specifying the shading technique that should be used to fill facets
    #: for geometric bodies when **renderStyle** =SHADED. The lighting of facets for meshed
    #: geometry is not affected by this setting. Possible values are:GOURAUD, specifying that
    #: lighting values should be computed for each corner of a facet and interpolated to fill
    #: the remainder of the facet.PHONG, specifying that lighting values should be computed for
    #: each pixel of a facet.The default value is GOURAUD.When set to **shading** =GOURAUD,
    #: lighting values are computed for each corner of a facet and the lighting for the
    #: remainder of the facet is interpolated from the corner values. When **shading** =PHONG,
    #: lighting values are computed for each pixel of a facet. Note: The shading technique can
    #: only be set to PHONG when the **shadersAvailable** member of GraphicsOptions is
    #: True.Printed output will only contain Phong shading if the output format is raster and
    #: the **accelerateOffScreen** member of GraphicsOptions is ON.The **renderStyle** setting is
    #: part of AssemblyDisplayOptions, PartDisplayOptions, and various other options objects
    #: not listed here since they are generally used for meshed geometry.
    shading: SymbolicConstant = GOURAUD

    #: A SymbolicConstant specifying how specular highlights should be calculated. Possible
    #: values are:INFINITE, specifying that a constant vector should be used for the direction
    #: from the camera to a vertex when computing specular highlights.LOCAL, specifying that
    #: the vector from the camera to each vertex should be calculated when computing specular
    #: highlights.The default value is INFINITE.When set to **viewpoint** =INFINITE, a constant
    #: vector is used for the direction from the camera (viewpoint) to a vertex. When
    #: **viewpoint** =LOCAL, the result is more realistic because the actual vector from the
    #: camera to each vertex is calculated. However, overall performance is decreased.
    viewpoint: SymbolicConstant = INFINITE

    #: A Float specifying the degree to which specular reflection is focused. The higher the
    #: **materialShininess** argument, the more focused the specular highlight. Possible values
    #: are 0.0 ≤ **polygonOffsetConstant** ≤ 128.0. The default value is 105.0.
    materialShininess: float = 105

    #: A LightArray object of length 8.
    lights: LightArray = []

    #: A String specifying the light applied evenly to the entire scene independent of any
    #: individual light. The initial value is 20% gray. A list of valid color strings is in the
    #: **colors** map in the **session** object.
    ambientColor: str = ""

    @abaqus_method_doc
    def setValues(
        self,
        shading: Literal[C.SHADED, C.PHONG, C.GOURAUD] = GOURAUD,
        viewpoint: Literal[C.LOCAL, C.INFINITE] = INFINITE,
        ambientColor: str = "",
        materialShininess: float = 105,
    ):
        """This method modifies the LightOptions object.

        Parameters
        ----------
        shading
            A SymbolicConstant specifying the shading technique that should be used to fill facets
            for geometric bodies when **renderStyle** =SHADED. The lighting of facets for meshed
            geometry is not affected by this setting. Possible values are:GOURAUD, specifying that
            lighting values should be computed for each corner of a facet and interpolated to fill
            the remainder of the facet.PHONG, specifying that lighting values should be computed for
            each pixel of a facet.The default value is GOURAUD.When set to **shading** =GOURAUD,
            lighting values are computed for each corner of a facet and the lighting for the
            remainder of the facet is interpolated from the corner values. When **shading** =PHONG,
            lighting values are computed for each pixel of a facet. Note: The shading technique can
            only be set to PHONG when the **shadersAvailable** member of GraphicsOptions is
            True.Printed output will only contain Phong shading if the output format is raster and
            the **accelerateOffScreen** member of GraphicsOptions is ON.The **renderStyle** setting is
            part of AssemblyDisplayOptions, PartDisplayOptions, and various other options objects
            not listed here since they are generally used for meshed geometry.
        viewpoint
            A SymbolicConstant specifying how specular highlights should be calculated. Possible
            values are:INFINITE, specifying that a constant vector should be used for the direction
            from the camera to a vertex when computing specular highlights.LOCAL, specifying that
            the vector from the camera to each vertex should be calculated when computing specular
            highlights.The default value is INFINITE.When set to **viewpoint** =INFINITE, a constant
            vector is used for the direction from the camera (viewpoint) to a vertex. When
            **viewpoint** =LOCAL, the result is more realistic because the actual vector from the
            camera to each vertex is calculated. However, overall performance is decreased.
        ambientColor
            A String specifying the light applied evenly to the entire scene independent of any
            individual light. The initial value is 20% gray. A list of valid color strings is in the
            **colors** map in the **session** object.
        materialShininess
            A Float specifying the degree to which specular reflection is focused. The higher the
            **materialShininess** argument, the more focused the specular highlight. Possible values
            are 0.0 ≤ **polygonOffsetConstant** ≤ 128.0. The default value is 105.0.

        Raises
        ------
        RangeError
        """
        ...



from __future__ import annotations

from .FXColor import FXColor
from .FXComposite import FXComposite
from .FXFont import FXFont
from .FXIcon import FXIcon
from .FXObject import FXObject
from .FXScrollArea import FXScrollArea
from .FXTreeItem import FXTreeItem

#: Extended selection mode allows for drag-selection of ranges of items.
TREELIST_EXTENDEDSELECT: int = hash("TREELIST_EXTENDEDSELECT")

#: Single selection mode allows up to one item to be selected.
TREELIST_SINGLESELECT: int = hash("TREELIST_SINGLESELECT")

#: Browse selection mode enforces one single item to be selected at all times.
TREELIST_BROWSESELECT: int = hash("TREELIST_BROWSESELECT")

#: Multiple selection mode is used for selection of individual items.
TREELIST_MULTIPLESELECT: int = hash("TREELIST_MULTIPLESELECT")

#: Automatically select under cursor.
TREELIST_AUTOSELECT: int = hash("TREELIST_AUTOSELECT")

#: Lines shown.
TREELIST_SHOWS_LINES: int = hash("TREELIST_SHOWS_LINES")

#: Boxes to expand shown.
TREELIST_SHOWS_BOXES: int = hash("TREELIST_SHOWS_BOXES")

#: Display root boxes also.
TREELIST_ROOT_BOXES: int = hash("TREELIST_ROOT_BOXES")

#: Display check boxes.
TREELIST_CHECK_BOXES: int = hash("TREELIST_CHECK_BOXES")

#: Propagate checked state to children and parents.
TREELIST_PROPAGATE_CHECKS: int = hash("TREELIST_PROPAGATE_CHECKS")

#: Normal tree list style.
TREELIST_NORMAL: int = hash("TREELIST_NORMAL")


class FXTreeList(FXScrollArea):
    """Tree list Widget."""

    def __init__(
        self,
        p: FXComposite,
        nvis: int,
        tgt: FXObject | None = None,
        sel: int = 0,
        opts: int = TREELIST_NORMAL,
        x: int = 0,
        y: int = 0,
        w: int = 0,
        h: int = 0,
    ):
        """Construct a tree list with nvis visible items; the tree list is initially empty.

        Parameters
        ----------
        p : FXComposite

        nvis : int

        tgt : FXObject | None

        sel : int

        opts : int

        x : int

        y : int

        w : int

        h : int
        """

    def addItemAfter(self, other: FXTreeItem, item: FXTreeItem, notify: bool = False):
        """Append new [possibly subclassed] item after to other item.

        Parameters
        ----------
        other : FXTreeItem

        item : FXTreeItem

        notify : bool
        """

    def addItemBefore(self, other: FXTreeItem, item: FXTreeItem, notify: bool = False):
        """Prepend new [possibly subclassed] item prior to other item.

        Parameters
        ----------
        other : FXTreeItem

        item : FXTreeItem

        notify : bool
        """

    def addItemFirst(self, p: FXTreeItem, item: FXTreeItem, notify: bool = False):
        """Prepend new [possibly subclassed] item as first child of p.

        Parameters
        ----------
        p : FXTreeItem

        item : FXTreeItem

        notify : bool
        """

    def addItemLast(self, p: FXTreeItem, item: FXTreeItem, notify: bool = False):
        """Append new [possibly subclassed] item as last child of p.

        Parameters
        ----------
        p : FXTreeItem

        item : FXTreeItem

        notify : bool
        """

    def canFocus(self):
        """Tree list can receive focus.

        Reimplemented from FXWindow.
        """

    def clearItems(self, notify: bool = False):
        """Remove all items from list.

        Parameters
        ----------
        notify : bool
        """

    def closeItem(self, item: FXTreeItem, notify: bool = False):
        """Close item.

        Parameters
        ----------
        item : FXTreeItem

        notify : bool
        """

    def collapseTree(self, tree: FXTreeItem, notify: bool = False):
        """Collapse tree.

        Parameters
        ----------
        tree : FXTreeItem

        notify : bool
        """

    def create(self):
        """Create server-side resources.

        Reimplemented from FXComposite. Reimplemented in FXDirList.
        """

    def deselectItem(self, item: FXTreeItem, notify: bool = False):
        """Deselect item.

        Parameters
        ----------
        item : FXTreeItem

        notify : bool
        """

    def destroy(self):
        """Destroy server-side resources.

        Reimplemented from FXComposite. Reimplemented in FXDirList.
        """

    def detach(self):
        """Detach server-side resources.

        Reimplemented from FXComposite. Reimplemented in FXDirList.
        """

    def disableItem(self, item: FXTreeItem):
        """Disable item.

        Parameters
        ----------
        item : FXTreeItem
        """

    def enableItem(self, item: FXTreeItem):
        """Enable item.

        Parameters
        ----------
        item : FXTreeItem
        """

    def expandTree(self, tree: FXTreeItem, notify: bool = False):
        """Expand tree.

        Parameters
        ----------
        tree : FXTreeItem

        notify : bool
        """

    def extendSelection(self, item: FXTreeItem, notify: bool = False):
        """Extend selection from anchor item to item.

        Parameters
        ----------
        item : FXTreeItem

        notify : bool
        """

    def findItem(self, text: str, start: FXTreeItem | None = None):
        """Search items for item by name, starting from start item; the flags argument controls the search
        direction, and case sensitivity.

        Parameters
        ----------
        text : str

        start : FXTreeItem | None
        """

    def getAnchorItem(self):
        """Return anchor item, if any."""

    def getContentHeight(self):
        """Return content height.

        Reimplemented from FXScrollArea.
        """

    def getContentWidth(self):
        """Compute and return content width.

        Reimplemented from FXScrollArea.
        """

    def getCurrentItem(self):
        """Return current item, if any."""

    def getCursorItem(self):
        """Return item under cursor, if any."""

    def getDefaultHeight(self):
        """Return default height.

        Reimplemented from FXScrollArea.
        """

    def getDefaultWidth(self):
        """Return default width.

        Reimplemented from FXScrollArea.
        """

    def getFirstItem(self):
        """REturn first root item."""

    def getFont(self):
        """Return text font."""

    def getHelpText(self):
        """Get the status line help text for this list."""

    def getIndent(self):
        """Return parent-child indent amount."""

    def getItemAt(self, x: int, y: int):
        """Get item at x,y, if any.

        Parameters
        ----------
        x : int

        y : int
        """

    def getItemCheck(self, item: FXTreeItem):
        """Returns the item checked state.

        Parameters
        ----------
        item : FXTreeItem
        """

    def getItemClosedIcon(self, item: FXTreeItem):
        """Return item's closed icon.

        Parameters
        ----------
        item : FXTreeItem
        """

    def getItemData(self, item: FXTreeItem):
        """Return item user-data pointer.

        Parameters
        ----------
        item : FXTreeItem
        """

    def getItemHeight(self, item: FXTreeItem):
        """Return item height.

        Parameters
        ----------
        item : FXTreeItem
        """

    def getItemOpenIcon(self, item: FXTreeItem):
        """Return item's open icon.

        Parameters
        ----------
        item : FXTreeItem
        """

    def getItemText(self, item: FXTreeItem):
        """Return item's text.

        Parameters
        ----------
        item : FXTreeItem
        """

    def getItemWidth(self, item: FXTreeItem):
        """Return item width.

        Parameters
        ----------
        item : FXTreeItem
        """

    def getLastItem(self):
        """Return last root item."""

    def getLineColor(self):
        """Return line color."""

    def getListStyle(self):
        """Return list style."""

    def getNumItems(self):
        """Return number of items."""

    def getNumVisible(self):
        """Return number of visible items."""

    def getSelBackColor(self):
        """Return selected text background."""

    def getSelTextColor(self):
        """Return selected text color."""

    def getTextColor(self):
        """Return normal text color."""

    def hitItem(self, item: FXTreeItem, x: int, y: int):
        """Return item hit code: 0 outside, 1 icon, 2 text, 3 box.

        Parameters
        ----------
        item : FXTreeItem

        x : int

        y : int

        """

    def isItemCurrent(self, item: FXTreeItem):
        """Return True if item is current.

        Parameters
        ----------
        item : FXTreeItem
        """

    def isItemEnabled(self, item: FXTreeItem):
        """Return True if item is enabled.

        Parameters
        ----------
        item : FXTreeItem
        """

    def isItemExpanded(self, item: FXTreeItem):
        """Return True if item expanded.

        Parameters
        ----------
        item : FXTreeItem
        """

    def isItemLeaf(self, item: FXTreeItem):
        """Return True if item is a leaf-item, i.e. has no children.

        Parameters
        ----------
        item : FXTreeItem
        """

    def isItemOpened(self, item: FXTreeItem):
        """Return True if item opened.

        Parameters
        ----------
        item : FXTreeItem
        """

    def isItemSelected(self, item: FXTreeItem):
        """Return True if item is selected.

        Parameters
        ----------
        item : FXTreeItem
        """

    def isItemVisible(self, item: FXTreeItem):
        """Return True if item is visible.

        Parameters
        ----------
        item : FXTreeItem
        """

    def killFocus(self):
        """Remove the focus from this window.

        Reimplemented from FXWindow.
        """

    def killSelection(self, notify: bool = False):
        """Deselect all items.

        Parameters
        ----------
        notify : bool
        """

    def makeItemVisible(self, item: FXTreeItem):
        """Scroll to make item visible.

        Parameters
        ----------
        item : FXTreeItem
        """

    def moveItemAfter(self, other: FXTreeItem, item: FXTreeItem):
        """Move item after other.

        Parameters
        ----------
        other : FXTreeItem

        item : FXTreeItem
        """

    def moveItemBefore(self, other: FXTreeItem, item: FXTreeItem):
        """Move item before other.

        Parameters
        ----------
        other : FXTreeItem

        item : FXTreeItem
        """

    def openItem(self, item: FXTreeItem, notify: bool = False):
        """Open item.

        Parameters
        ----------
        item : FXTreeItem

        notify : bool
        """

    def recalc(self):
        """Recalculate layout.

        Reimplemented from FXWindow.
        """

    def removeItem(self, item: FXTreeItem, notify: bool = False):
        """Remove item.

        Parameters
        ----------
        item : FXTreeItem

        notify : bool
        """

    def removeItems(self, fm: FXTreeItem, to: FXTreeItem, notify: bool = False):
        """Remove items in range [fm, to] inclusively.

        Parameters
        ----------
        fm : FXTreeItem

        to : FXTreeItem

        notify : bool
        """

    def reparentItem(self, item: FXTreeItem, p: FXTreeItem):
        """Reparent item under parent p.

        Parameters
        ----------
        item : FXTreeItem

        p : FXTreeItem
        """

    def selectItem(self, item: FXTreeItem, notify: bool = False):
        """Select item.

        Parameters
        ----------
        item : FXTreeItem

        notify : bool
        """

    def setAnchorItem(self, item: FXTreeItem):
        """Change anchor item.

        Parameters
        ----------
        item : FXTreeItem
        """

    def setCurrentItem(self, item: FXTreeItem, notify: bool = False):
        """Change current item.

        Parameters
        ----------
        item : FXTreeItem

        notify : bool
        """

    def setFocus(self):
        """Move the focus to this window.

        Reimplemented from FXWindow.
        """

    def setFont(self, fnt: FXFont):
        """Change text font.

        Parameters
        ----------
        fnt : FXFont
        """

    def setHelpText(self, text: str):
        """Set the status line help text for this list.

        Parameters
        ----------
        text : str
        """

    def setIndent(self, in_: int):
        """Change parent-child indent amount.

        Parameters
        ----------
        in_ : int
        """

    def setItemCheck(self, item: FXTreeItem, check: int, notify: bool = False):
        """Sets the item checked state. Valid states are True, False and MAYBE. Returns True if the check value
        has changed, False otherwise.

        Parameters
        ----------
        item : FXTreeItem

        check : int

        notify : bool
        """

    def setItemClosedIcon(self, item: FXTreeItem, icon: FXIcon):
        """Chance item's closed icon.

        Parameters
        ----------
        item : FXTreeItem

        icon : FXIcon
        """

    def setItemData(self, item: FXTreeItem, ptr: str):
        """Change item user-data pointer.

        Parameters
        ----------
        item : FXTreeItem

        ptr : str
        """

    def setItemOpenIcon(self, item: FXTreeItem, icon: FXIcon):
        """Change item's open icon.

        Parameters
        ----------
        item : FXTreeItem

        icon : FXIcon
        """

    def setItemText(self, item: FXTreeItem, text: str):
        """Change item's text.

        Parameters
        ----------
        item : FXTreeItem

        text : str
        """

    def setLineColor(self, clr: FXColor):
        """Change line color.

        Parameters
        ----------
        clr : FXColor
        """

    def setListStyle(self, style: int):
        """Change list style.

        Parameters
        ----------
        style : int
        """

    def setNumVisible(self, nvis: int):
        """Change number of visible items.

        Parameters
        ----------
        nvis : int
        """

    def setSelBackColor(self, clr: FXColor):
        """Change selected text background.

        Parameters
        ----------
        clr : FXColor
        """

    def setSelTextColor(self, clr: FXColor):
        """Change selected text color.

        Parameters
        ----------
        clr : FXColor
        """

    def setTextColor(self, clr: FXColor):
        """Change normal text color.

        Parameters
        ----------
        clr : FXColor
        """

    def toggleItem(self, item: FXTreeItem, notify: bool = False):
        """Toggle item selection.

        Parameters
        ----------
        item : FXTreeItem

        notify : bool
        """

    def updateItem(self, item: FXTreeItem):
        """Repaint item.

        Parameters
        ----------
        item : FXTreeItem
        """



from __future__ import annotations

from abqpy.decorators import abaqus_class_doc, abaqus_function_doc

from ..Odb.Odb import Odb
from ..Part.Part import Part
from ..Region.Region import Region


@abaqus_class_doc
class PlyStackPlot:
    """The PlyStackPlot object is used to plot the stacking of plies in a composite layup or in a composite
    shell section.

    .. note::
        This object can be accessed by::

            import section
            import visualization
    """


@abaqus_function_doc
def MdbPlyStackPlot(part: Part, region: Region) -> PlyStackPlot:
    """This method creates a PlyStackPlot object from a region of a part that contains a composite shell layup.

    .. note::
        This function can be accessed by::

            section.MdbPlyStackPlot

    Parameters
    ----------
    part
        A Part object.
    region
        A Region object which contains a composite shell layup.

    Returns
    -------
    PlyStackPlot
        A PlyStackPlot object.

    Raises
    ------
        None.
    """
    return PlyStackPlot()


@abaqus_function_doc
def OdbPlyStackPlot(odb: Odb, sectionName: str, offset: float = 0):
    """This method creates a PlyStackPlot object from a composite shell section of an Odb object.

    .. note::
        This function can be accessed by::

            visualization.OdbPlyStackPlot

    Parameters
    ----------
    odb
        An Odb object.
    sectionName
        A String specifying the section name that contains a composite shell section.
    offset
        A Float specifying the shell offset. The default value is 0.0.

    Returns
    -------
       A PlyStackPlot object.

    Raises
    ------
        None.
    """
    return PlyStackPlot()



from __future__ import annotations

from abqpy.decorators import abaqus_class_doc, abaqus_method_doc

from ..UtilityAndView.abaqusConstants import OFF, Boolean
from .ContactProperty import ContactProperty


@abaqus_class_doc
class CavityRadiationProp(ContactProperty):
    """The CavityRadiationProp object is an interaction property that defines emissivity as a function of
    temperature and field variables. The CavityRadiationProp object is derived from the InteractionProperty
    object.

    .. note::
        This object can be accessed by::

            import interaction
            mdb.models[name].interactionProperties[name]

        The corresponding analysis keywords are:

        - EMISSIVITY
    """

    #: A String specifying the interaction property repository key.
    name: str

    #: A Boolean specifying whether the data depend on temperature. The default value is OFF.
    temperatureDependency: Boolean = OFF

    #: An Int specifying the number of field variable dependencies. The default value is 0.
    dependencies: int = 0

    #: A sequence of sequences of Floats specifying the following:The emissivity,
    #: ϵϵ.Temperature, if the data depend on temperature.Value of the first field variable, if
    #: the data depend on field variables.Value of the second field variable.Etc.
    property: tuple[tuple[float, ...], ...] = ()

    @abaqus_method_doc
    def __init__(
        self,
        name: str,
        temperatureDependency: Boolean = OFF,
        dependencies: int = 0,
        property: tuple = (),
    ):
        """This method creates a CavityRadiationProp object.

        .. note::
            This function can be accessed by::

                mdb.models[name].CavityRadiationProp

        Parameters
        ----------
        name
            A String specifying the interaction property repository key.
        temperatureDependency
            A Boolean specifying whether the data depend on temperature. The default value is OFF.
        dependencies
            An Int specifying the number of field variable dependencies. The default value is 0.
        property
            A sequence of sequences of Floats specifying the following:The emissivity,
            ϵϵ.Temperature, if the data depend on temperature.Value of the first field variable, if
            the data depend on field variables.Value of the second field variable.Etc.

        Returns
        -------
        CavityRadiationProp
            A CavityRadiationProp object.
        """
        super().__init__(name)

    @abaqus_method_doc
    def setValues(
        self,
        temperatureDependency: Boolean = OFF,
        dependencies: int = 0,
        property: tuple = (),
    ):
        """This method modifies the CavityRadiationProp object.

        Parameters
        ----------
        temperatureDependency
            A Boolean specifying whether the data depend on temperature. The default value is OFF.
        dependencies
            An Int specifying the number of field variable dependencies. The default value is 0.
        property
            A sequence of sequences of Floats specifying the following:The emissivity,
            ϵϵ.Temperature, if the data depend on temperature.Value of the first field variable, if
            the data depend on field variables.Value of the second field variable.Etc.
        """
        ...



from __future__ import annotations

from .FXObject import FXObject


class FXDataTarget(FXObject):
    """A Data Target allows a valuator widget such as a Slider or Text Field to be directly connected with a
    variable in the program.

    Whenever the valuator control changes, the variable connected through the data target is automatically
    updated; conversely, whenever the program changes a variable, all the connected valuator widgets will be
    updated to reflect this new value on the display. Data Targets also allow connecting Radio Buttons, Menu
    Commands, and so on to a variable. In this case, the new value of the connected variable is computed by
    substracting ID_OPTION from the message ID.
    """

    #: Will cause the FXDataTarget to ask sender for value.
    ID_VALUE: int = hash("ID_VALUE")

    #: ID_OPTION+i will set the value to i where -10000<=i<=10000.
    ID_OPTION: int = hash("ID_OPTION")

    def __init__(self, value: str, tgt: FXObject | None = None, sel: int = 0):
        """Associate with string variable.

        Parameters
        ----------
        value : str

        tgt : FXObject | None

        sel : int
        """

    def connect(self):
        """Associate with nothing."""

    def getData(self):
        """Return pointer to data its connected to."""

    def getSelector(self):
        """Get the message identifier for this data target."""

    def getTarget(self):
        """Get the message target object for this data target, if any."""

    def getType(self):
        """Return type of data its connected to."""

    def setSelector(self, sel: int):
        """Set the message identifier for this data target.

        Parameters
        ----------
        sel : int
        """

    def setTarget(self, t: FXObject):
        """Set the message target object for this data target.

        Parameters
        ----------
        t : FXObject
        """



from __future__ import annotations

from abqpy.decorators import abaqus_class_doc, abaqus_method_doc

from ...UtilityAndView.abaqusConstants import OFF, Boolean


@abaqus_class_doc
class GasketMembraneElastic:
    r"""The GasketMembraneElastic object defines the elastic parameters for the membrane shear behavior of a
    gasket.

    .. note::
        This object can be accessed by::

            import material
            mdb.models[name].materials[name].gasketMembraneElastic
            import odbMaterial
            session.odbs[name].materials[name].gasketMembraneElastic

        The table data for this object are:

        - Young's modulus, :math:`E`.
        - Poisson's ratio, :math:`\nu`.
        - Temperature, if the data depend on temperature.
        - Value of the first field variable, if the data depend on field variables.
        - Value of the second field variable.
        - Etc.

        The corresponding analysis keywords are:

        - GASKET ELASTICITY
    """

    @abaqus_method_doc
    def __init__(self, table: tuple, temperatureDependency: Boolean = OFF, dependencies: int = 0):
        """This method creates a GasketMembraneElastic object.

        .. note::
            This function can be accessed by::

                mdb.models[name].materials[name].GasketMembraneElastic
                session.odbs[name].materials[name].GasketMembraneElastic

        Parameters
        ----------
        table
            A sequence of sequences of Floats specifying the items described below.
        temperatureDependency
            A Boolean specifying whether the data depend on temperature. The default value is OFF.
        dependencies
            An Int specifying the number of field variable dependencies. The default value is 0.

        Returns
        -------
        GasketMembraneElastic
            A GasketMembraneElastic object.

        Raises
        ------
        RangeError
        """
        ...

    @abaqus_method_doc
    def setValues(self, *args, **kwargs):
        """This method modifies the GasketMembraneElastic object.

        Raises
        ------
        RangeError
        """
        ...



from __future__ import annotations

from typing import TYPE_CHECKING

from typing_extensions import Literal

from abqpy.decorators import abaqus_class_doc, abaqus_method_doc

from ..Adaptivity.AdaptiveMeshConstraint import AdaptiveMeshConstraint
from ..Adaptivity.AdaptiveMeshControl import AdaptiveMeshControl
from ..Adaptivity.RemeshingRule import RemeshingRule
from ..Amplitude.Amplitude import Amplitude
from ..BeamSectionProfile.Profile import Profile
from ..BoundaryCondition.BoundaryCondition import BoundaryCondition
from ..Calibration.Calibration import Calibration
from ..Constraint.Constraint import Constraint
from ..Feature.FeatureOptions import FeatureOptions
from ..Field.AnalyticalField import AnalyticalField
from ..Field.DiscreteField import DiscreteField
from ..Filter.Filter import Filter
from ..Interaction.ContactControl import ContactControl
from ..Interaction.ContactInitialization import ContactInitialization
from ..Interaction.ContactProperty import ContactProperty
from ..Interaction.ContactStabilization import ContactStabilization
from ..Interaction.Interaction import Interaction
from ..Load.Load import Load
from ..Material.Material import Material
from ..Optimization.OptimizationTask import OptimizationTask
from ..Part.Part import Part
from ..PredefinedField.PredefinedField import PredefinedField
from ..Section.Section import Section
from ..Sketcher.ConstrainedSketch import ConstrainedSketch
from ..Step.InitialStep import InitialStep
from ..Step.Step import Step
from ..StepOutput.FieldOutputRequest import FieldOutputRequest
from ..StepOutput.HistoryOutputRequest import HistoryOutputRequest
from ..StepOutput.IntegratedOutputSection import IntegratedOutputSection
from ..StepOutput.TimePoint import TimePoint
from ..UtilityAndView.abaqusConstants import (
    NOT_SET,
    OFF,
    ON,
    STANDARD_EXPLICIT,
    Boolean,
    SymbolicConstant,
)
from ..UtilityAndView.abaqusConstants import abaqusConstants as C
from .KeywordBlock import KeywordBlock

if TYPE_CHECKING:
    from ..Assembly.Assembly import Assembly


@abaqus_class_doc
class ModelBase:
    """Abaqus creates a Model object named `Model-1` when a session is started.

    .. note::
        This object can be accessed by::

            mdb.models[name]

        The corresponding analysis keywords are:

        - PHYSICAL CONSTANTS
    """

    #: A String specifying the repository key.
    name: str = ""

    #: None or a Float specifying the Stefan-Boltzmann constant. The default value is None.
    stefanBoltzmann: float | None = None

    #: None or a Float specifying the absolute zero constant. The default value is None.
    absoluteZero: float | None = None

    #: A SymbolicConstant specifying the type of incident wave formulation to be used in
    #: acoustic problems. Possible values are NOT_SET, SCATTERED, and TOTAL. The default value
    #: is NOT_SET.
    waveFormulation: SymbolicConstant = NOT_SET

    #: None or a Float specifying the universal gas constant. The default value is None.
    universalGas: float | None = None

    #: A Boolean specifying whether an input file should be written without parts and
    #: assemblies. The default value is OFF.
    noPartsInputFile: Boolean = OFF

    #: An Int specifying the increment, interval, iteration or cycle where the restart analysis
    #: will start. To select the end of the step use the SymbolicConstant STEP_END.
    restartIncrement: int | SymbolicConstant

    #: A Boolean specifying that the step specified by **restartStep** should be terminated at
    #: the increment specified by **restartIncrement**.
    endRestartStep: Boolean = OFF

    #: A Boolean specifying that a shell global model drives a solid submodel.
    shellToSolid: Boolean = OFF

    #: A Float specifying the time stamp that indicates when the model was last changed.
    lastChangedCount: float | None = None

    #: A String specifying the purpose and contents of the Model object. The default value is
    #: an empty string.
    description: str = ""

    #: A String specifying the name of the job that generated the restart data.
    restartJob: str = ""

    #: A String specifying the name of the step where the restart analysis will start.
    restartStep: str = ""

    #: A String specifying the name of the job that generated the results for the global model.
    globalJob: str = ""

    #: A boolean specifying the status of constraints created in a model, in the model which
    #: instances this model.
    copyConstraints: Boolean = OFF

    #: A boolean specifying the status of connectors created in a model, in the model which
    #: instances this model.
    copyConnectors: Boolean = OFF

    #: A boolean specifying the status of interactions created in a model, in the model which
    #: instances this model.
    copyInteractions: Boolean = OFF

    #: A KeywordBlock object.
    keywordBlock: KeywordBlock = KeywordBlock()

    #: An Assembly object.
    rootAssembly: Assembly

    #: A repository of Amplitude objects.
    amplitudes: dict[str, Amplitude] = {}

    #: A repository of Profile objects.
    profiles: dict[str, Profile] = {}

    #: A repository of BoundaryCondition objects.
    boundaryConditions: dict[str, BoundaryCondition] = {}

    #: A repository of ConstrainedSketchConstraint objects.
    constraints: dict[str, Constraint] = {}

    #: A repository of AnalyticalField objects.
    analyticalFields: dict[str, AnalyticalField] = {}

    #: A repository of DiscreteField objects.
    discreteFields: dict[str, DiscreteField] = {}

    #: A repository of PredefinedField objects.
    predefinedFields: dict[str, PredefinedField] = {}

    #: A repository of Interaction objects.
    interactions: dict[str, Interaction] = {}

    #: A repository of InteractionProperty objects.
    interactionProperties: dict[str, ContactProperty] = {}

    #: A repository of ContactControl objects.
    contactControls: dict[str, ContactControl] = {}

    #: A repository of ContactInitialization objects.
    contactInitializations: dict[str, ContactInitialization] = {}

    #: A repository of ContactStabilization objects.
    contactStabilizations: dict[str, ContactStabilization] = {}

    #: A tuple of tuples of Strings specifying the linked child PartInstance name in the
    #: current model to the corresponding parent PartInstance name in a different model.
    linkedInstances: tuple = ()

    #: A tuple of tuples of Strings specifying the linked child Part name in the current model
    #: to the corresponding parent Part name in a different model.
    linkedParts: tuple = ()

    #: A repository of Load objects.
    loads: dict[str, Load] = {}

    #: A repository of Material objects.
    materials: dict[str, Material] = {}

    #: A repository of Calibration objects.
    calibrations: dict[str, Calibration] = {}

    #: A repository of Section objects.
    sections: dict[str, Section] = {}

    #: A repository of RemeshingRule objects.
    remeshingRules: dict[str, RemeshingRule] = {}

    #: A repository of ConstrainedSketch objects.
    sketches: dict[str, ConstrainedSketch] = {}

    #: A repository of Part objects.
    parts: dict[str, Part] = {}

    #: A repository of Step objects.
    steps: dict[str, Step] = {}

    #: A FeatureOptions object.
    featureOptions: FeatureOptions = FeatureOptions()

    #: A repository of AdaptiveMeshConstraint objects.
    adaptiveMeshConstraints: dict[str, AdaptiveMeshConstraint] = {}

    #: A repository of AdaptiveMeshControl objects.
    adaptiveMeshControls: dict[str, AdaptiveMeshControl] = {}

    #: A repository of TimePoint objects.
    timePoints: dict[str, TimePoint] = {}

    #: A repository of Filter objects.
    filters: dict[str, Filter] = {}

    #: A repository of IntegratedOutputSection objects.
    integratedOutputSections: dict[str, IntegratedOutputSection] = {}

    #: A repository of FieldOutputRequest objects.
    fieldOutputRequests: dict[str, FieldOutputRequest] = {}

    #: A repository of HistoryOutputRequest objects.
    historyOutputRequests: dict[str, HistoryOutputRequest] = {}

    #: A repository of OptimizationTask objects.
    optimizationTasks: dict[str, OptimizationTask] = {}

    @abaqus_method_doc
    def __init__(
        self,
        name: str,
        description: str = "",
        stefanBoltzmann: float | None = None,
        absoluteZero: float | None = None,
        waveFormulation: Literal[C.SCATTERED, C.NOT_SET, C.TOTAL] = NOT_SET,
        modelType: Literal[C.STANDARD_EXPLICIT, C.ELECTROMAGNETIC] = STANDARD_EXPLICIT,
        universalGas: float | None = None,
        copyConstraints: Boolean = ON,
        copyConnectors: Boolean = ON,
        copyInteractions: Boolean = ON,
    ):
        """This method creates a Model object.

        .. note::
            This function can be accessed by::

                mdb.Model

        Parameters
        ----------
        name
            A String specifying the repository key.
        description
            A String specifying the purpose and contents of the Model object. The default value is
            an empty string.
        stefanBoltzmann
            None or a Float specifying the Stefan-Boltzmann constant. The default value is None.
        absoluteZero
            None or a Float specifying the absolute zero constant. The default value is None.
        waveFormulation
            A SymbolicConstant specifying the type of incident wave formulation to be used in
            acoustic problems. Possible values are NOT_SET, SCATTERED, and TOTAL. The default value
            is NOT_SET.
        modelType
            A SymbolicConstant specifying the analysis model type. Possible values are
            STANDARD_EXPLICIT and ELECTROMAGNETIC. The default is STANDARD_EXPLICIT.
        universalGas
            None or a Float specifying the universal gas constant. The default value is None.
        copyConstraints
            A boolean specifying whether to copy the constraints created in the model to the model
            that instances this model. The default value is ON.
        copyConnectors
            A boolean specifying whether to copy the connectors created in the model to the model
            that instances this model. The default value is ON.
        copyInteractions
            A boolean specifying whether to copy the interactions created in the model to the model
            that instances this model. The default value is ON.

        Returns
        -------
        Model
            A Model object.
        """
        from ..Assembly.Assembly import Assembly

        self.steps["Initial"] = InitialStep()
        self.rootAssembly = Assembly()

    @abaqus_method_doc
    def setValues(
        self,
        description: str = "",
        noPartsInputFile: Boolean = OFF,
        absoluteZero: float | None = None,
        stefanBoltzmann: float | None = None,
        waveFormulation: Literal[C.SCATTERED, C.NOT_SET, C.TOTAL] = NOT_SET,
        universalGas: float | None = None,
        restartJob: str = "",
        restartStep: str = "",
        restartIncrement: Literal[C.STEP_END] | None = None,
        endRestartStep: Boolean = OFF,
        globalJob: str = "",
        shellToSolid: Boolean = OFF,
        copyConstraints: Boolean = OFF,
        copyConnectors: Boolean = OFF,
        copyInteractions: Boolean = OFF,
    ):
        """This method modifies the Model object.

        Parameters
        ----------
        description
            A String specifying the purpose and contents of the Model object. The default value is
            an empty string.
        noPartsInputFile
            A Boolean specifying whether an input file should be written without parts and
            assemblies. The default value is OFF.
        absoluteZero
            None or a Float specifying the absolute zero constant. The default value is None.
        stefanBoltzmann
            None or a Float specifying the Stefan-Boltzmann constant. The default value is None.
        waveFormulation
            A SymbolicConstant specifying the type of incident wave formulation to be used in
            acoustic problems. Possible values are NOT_SET, SCATTERED, and TOTAL. The default value
            is NOT_SET.
        universalGas
            None or a Float specifying the universal gas constant. The default value is None.
        restartJob
            A String specifying the name of the job that generated the restart data.
        restartStep
            A String specifying the name of the step where the restart analysis will start.
        restartIncrement
            An Int specifying the increment, interval, iteration or cycle where the restart analysis
            will start. To select the end of the step use the SymbolicConstant STEP_END.
        endRestartStep
            A Boolean specifying that the step specified by **restartStep** should be terminated at
            the increment specified by **restartIncrement**.
        globalJob
            A String specifying the name of the job that generated the results for the global model.
        shellToSolid
            A Boolean specifying that a shell global model drives a solid submodel.
        copyConstraints
            A Boolean specifying whether to copy the constraints created in the model to the model
            that instances this model.
        copyConnectors
            A Boolean specifying whether to copy the connectors created in the model to the model
            that instances this model
        copyInteractions
            A Boolean specifying whether to copy the interactions created in the model to the model
            that instances this model.
        """
        ...



from __future__ import annotations

from abqpy.decorators import abaqus_class_doc, abaqus_method_doc

from ..UtilityAndView.abaqusConstants import DEFAULT_MODEL, SymbolicConstant
from .Leaf import Leaf


@abaqus_class_doc
class LeafFromModelElemLabels(Leaf):
    """The LeafFromModelElemLabels object can be used whenever a Leaf object is expected as an argument. Leaf
    objects are used to specify the items in a display group. Leaf objects are constructed as temporary objects,
    which are then used as arguments to DisplayGroup commands. The LeafFromModelElemLabels object is derived
    from the Leaf object.

    .. note::
        This object can be accessed by::

            import displayGroupOdbToolset
    """

    #: A SymbolicConstant specifying the leaf type. Possible values are EMPTY_LEAF,
    #: DEFAULT_MODEL, ALL_ELEMENTS, ALL_NODES, and ALL_SURFACES.
    leafType: SymbolicConstant

    #: A sequence of Strings specifying expressions that denote element labels per part
    #: instance in the model. Each part instance element expression is a sequence of a String
    #: specifying the part instance name and a sequence of element expressions; for example,
    #: `(('partInstance1',(1,'7','3:15;3'),), ('partInstance2','8'),))`. The element
    #: expressions can be any of the following:An Int specifying a single element label; for
    #: example, `1`.A String specifying a single element label; for example, `'7'`.A String
    #: specifying a sequence of element labels; for example, `'3:5'` and `'3:15:3'`.
    elementLabels: tuple[str, ...] = ()

    @abaqus_method_doc
    def __init__(self, elementLabels: tuple):
        """This method creates a Leaf object from a sequence of element labels spanning several part instances.

        .. note::
            This function can be accessed by::

                LeafFromModelElemLabels

        Parameters
        ----------
        elementLabels
            A sequence of Strings specifying expressions that denote element labels per part
            instance in the model. Each part instance element expression is a sequence of a String
            specifying the part instance name and a sequence of element expressions; for example,
            `(('partInstance1',(1,'7','3:15;3'),), ('partInstance2','8'),))`. The element
            expressions can be any of the following:An Int specifying a single element label; for
            example, `1`.A String specifying a single element label; for example, `'7'`.A String
            specifying a sequence of element labels; for example, `'3:5'` and `'3:15:3'`.

        Returns
        -------
        LeafFromModelElemLabels
            A LeafFromModelElemLabels object.
        """
        super().__init__(DEFAULT_MODEL)



import auto_all

auto_all.start_all()

from abaqus.UtilityAndView.abaqusConstants import (  # noqa
    ABORTED,
    ANY_JOB,
    ANY_MESSAGE_TYPE,
    BATCHPRE_PHASE,
    CALCULATOR_PHASE,
    CFD_PHASE,
    CHECK_SUBMITTED,
    COMPLETED,
    END_STEP,
    ERROR,
    EXPLICIT_PHASE,
    HEADING,
    HEALER_JOB,
    HEALER_PHASE,
    HEALER_TYPE,
    INTERRUPTED,
    ITERATION,
    JOB_ABORTED,
    JOB_COMPLETED,
    JOB_INTERRUPTED,
    JOB_SUBMITTED,
    MONITOR_DATA,
    ODB_FILE,
    ODB_FRAME,
    PACKAGER_PHASE,
    SIMULATION_ABORTED,
    SIMULATION_COMPLETED,
    SIMULATION_INTERRUPTED,
    SIMULATION_STARTED,
    STANDARD_PHASE,
    STARTED,
    STATUS,
    STEP,
    U1,
    WARNING,
)

auto_all.end_all()



"""The following methods provide access to the standard message dialogs."""

from __future__ import annotations

from .FXObject import FXObject
from .FXWindow import FXWindow


def showAFXDismissableWarningDialog(owner: FXWindow, message: str, tgt: FXObject | None = None, sel: int = 0):
    """Posts a modal dismissable warning dialog box that has a "Show this dialog next time" check button, via
    which the user can request not to see a specific type of messages in the future. The dialog box has a
    getCheckButtonState method that can be called to get the state of the check button in the dialog box.

    Parameters
    ----------
    owner : FXWindow
        Window over which the dialog box is to be centered.
    message : str
        Text to be displayed in the dialog box.
    tgt : FXObject
        Message target.
    sel : int
        Message ID.
    """


def showAFXErrorDialog(owner: FXWindow, message: str, tgt: FXObject | None = None, sel: int = 0):
    """Shows an error dialog box.

    Parameters
    ----------
    owner : FXWindow
        Window over which the dialog box is to be centered.
    message : str
        Text to be displayed in the dialog box.
    tgt : FXObject
        Message target.
    sel : int
        Message ID.
    """


def showAFXItemsWarningDialog(
    owner: FXWindow, topMessage: str, items: str, bottomMessage: str, tgt: FXObject | None = None, sel: int = 0
):
    """Shows a modal warning dialog box that has a scrollable list of items and two messages: one placed above and one below the list. .

    Parameters
    ----------
    owner : FXWindow
        Window over which the dialog box is to be centered.
    topMessage : str
        Text to be displayed on top of the list of items.
    items : str
        A String containing a comma-separated list of items to be displayed between the topMessage and bottomMessage.
    bottomMessage : str
        Text to be displayed below the list of items.
    tgt : FXObject
        Message target.
    sel : int
        Message ID.
    """


def showAFXWarningDialog(owner: FXWindow, message: str, tgt: FXObject | None = None, sel: int = 0):
    """Shows a warning dialog box.

    Parameters
    ----------
    owner : FXWindow
        Window over which the dialog box is to be centered.
    message : str
        Text to be displayed in the dialog box.
    tgt : FXObject
        Message target.
    sel : int
        Message ID.
    """


def showAFXInformationDialog(owner: FXWindow, message: str, tgt: FXObject | None = None, sel: int = 0):
    """Shows an information dialog box.

    Parameters
    ----------
    owner : FXWindow
        Window over which the dialog box is to be centered.
    message : str
        Text to be displayed in the dialog box.
    tgt : FXObject
        Message target.
    sel : int
        Message ID.
    """



from __future__ import annotations

from .constants import DEFAULT_PAD
from .FXColor import FXColor
from .FXComposite import FXComposite
from .FXFont import FXFont
from .FXFrame import FXFrame
from .FXIcon import FXIcon

#: Icon appears under text.
ICON_UNDER_TEXT: int = hash("ICON_UNDER_TEXT")

#: Icon appears after text (to its right).
ICON_AFTER_TEXT: int = hash("ICON_AFTER_TEXT")

#: Icon appears before text (to its left).
ICON_BEFORE_TEXT: int = hash("ICON_BEFORE_TEXT")

#: Icon appears above text.
ICON_ABOVE_TEXT: int = hash("ICON_ABOVE_TEXT")

#: Icon appears below text.
ICON_BELOW_TEXT: int = hash("ICON_BELOW_TEXT")

#: Same as ICON_UNDER_TEXT.
TEXT_OVER_ICON: int = hash("TEXT_OVER_ICON")

#: Same as ICON_BEFORE_TEXT.
TEXT_AFTER_ICON: int = hash("TEXT_AFTER_ICON")

#: Same as ICON_AFTER_TEXT.
TEXT_BEFORE_ICON: int = hash("TEXT_BEFORE_ICON")

#: Same as ICON_BELOW_TEXT.
TEXT_ABOVE_ICON: int = hash("TEXT_ABOVE_ICON")

#: Same as ICON_ABOVE_TEXT.
TEXT_BELOW_ICON: int = hash("TEXT_BELOW_ICON")

#: Combination of JUSTIFY_NORMAL & ICON_BEFORE_TEXT.
LABEL_NORMAL: int = hash("LABEL_NORMAL")


class FXLabel(FXFrame):
    """A label widget can be used to place a text and/or icon for explanation purposes.

    The text label may have an optional tooltip and/or help string.
    """

    def __init__(
        self,
        p: FXComposite,
        text: str,
        ic: FXIcon | None = None,
        opts: int = LABEL_NORMAL,
        x: int = 0,
        y: int = 0,
        w: int = 0,
        h: int = 0,
        pl: int = DEFAULT_PAD,
        pr: int = DEFAULT_PAD,
        pt: int = DEFAULT_PAD,
        pb: int = DEFAULT_PAD,
    ):
        """Construct label with given text and icon.

        Parameters
        ----------
        p : FXComposite

        text : str

        ic : FXIcon | None

        opts : int

        x : int

        y : int

        w : int

        h : int

        pl : int

        pr : int

        pt : int

        pb : int
        """

    def create(self):
        """Create server-side resources.

        Reimplemented from FXWindow. Reimplemented in FXMenuButton, FXOptionMenu, FXToggleButton, and
        AFXFlyoutButton.
        """

    def detach(self):
        """Detach server-side resources.

        Reimplemented from FXWindow. Reimplemented in FXMenuButton, FXOptionMenu, FXToggleButton, and
        AFXFlyoutButton.
        """

    def disable(self):
        """Disable the window.

        Reimplemented from FXWindow. Reimplemented in AFXFlyoutButton.
        """

    def enable(self):
        """Enable the window.

        Reimplemented from FXWindow. Reimplemented in AFXFlyoutButton.
        """

    def getDefaultHeight(self):
        """Return default height.

        Reimplemented from FXFrame. Reimplemented in FXCheckButton, FXMDIDeleteButton, FXMDIRestoreButton,
        FXMDIMaximizeButton, FXMDIMinimizeButton, FXMDIWindowButton, FXMenuButton, FXOption, FXOptionMenu,
        FXRadioButton, and FXToggleButton.
        """

    def getDefaultWidth(self):
        """Return default width.

        Reimplemented from FXFrame. Reimplemented in FXCheckButton, FXMDIDeleteButton, FXMDIRestoreButton,
        FXMDIMaximizeButton, FXMDIMinimizeButton, FXMDIWindowButton, FXMenuButton, FXOption, FXOptionMenu,
        FXRadioButton, and FXToggleButton.
        """

    def getFont(self):
        """Get the text font."""

    def getHelpText(self):
        """Get the status line help text for this label."""

    def getIcon(self):
        """Get the icon for this label."""

    def getIconPosition(self):
        """Get the current icon position."""

    def getJustify(self):
        """Get the current text-justification mode."""

    def getText(self):
        """Get the text for this label."""

    def getTextColor(self):
        """Get the current text color."""

    def getTipText(self):
        """Get the tool tip message for this label."""

    def setFont(self, fnt: FXFont):
        """Set the text font.

        Parameters
        ----------
        fnt : FXFont
        """

    def setHelpText(self, text: str):
        """Set the status line help text for this label. Reimplemented in AFXFlyoutItem.

        Parameters
        ----------
        text : str
        """

    def setIcon(self, ic: FXIcon):
        """Set the icon for this label. Reimplemented in AFXFlyoutItem.

        Parameters
        ----------
        ic : FXIcon
        """

    def setIconPosition(self, mode: int):
        """Set the current icon position.

        Parameters
        ----------
        mode : int
        """

    def setJustify(self, mode: int):
        """Set the current text-justification mode.

        Parameters
        ----------
        mode : int
        """

    def setText(self, text: str):
        """Set the text for this label. Reimplemented in AFXFlyoutItem.

        Parameters
        ----------
        text : str
        """

    def setTextColor(self, clr: FXColor):
        """Set the current text color.

        Parameters
        ----------
        clr : FXColor
        """

    def setTipText(self, text: str):
        """Set the tool tip message for this label. Reimplemented in AFXFlyoutItem.

        Parameters
        ----------
        text : str
        """



from __future__ import annotations

from .AFXCommand import AFXCommand
from .FXDataTarget import FXDataTarget


class AFXKeyword(FXDataTarget):
    """Abaqus."""

    #: Used to activate the keyword.
    ID_ACTIVATE: int = hash("ID_ACTIVATE")

    #: Used to deactivate the keyword.
    ID_DEACTIVATE: int = hash("ID_DEACTIVATE")

    def __init__(self, command: AFXCommand, name: str, isRequired: bool = False):
        """Constructor.

        Parameters
        ----------
        command : AFXCommand
            Host command, or NULL to create a keyword not associated with a command.
        name : str
            Keyword name.
        isRequired : bool
            True if the keyword is a required argument of the command.
        """

    def activate(self):
        """Activates the keyword; active keywords will be processed during command generation."""

    def deactivate(self):
        """Deactivates the keyword; inactive keywords will not be processed during command generation."""

    def getCommandSnippet(self):
        """Returns the command snippet of the keyword based on its current value."""

    def getName(self):
        """Returns the keyword name."""

    def getSetupCommands(self):
        """Returns the keyword's variable initilization commands (part of the generated command string)."""

    def getTypeName(self):
        """Returns the keyword type name.

        Implemented in AFXBoolKeyword, AFXComSymConstKeyword, AFXComTableKeyword, AFXFloatKeyword,
        AFXIntKeyword, AFXObjectKeyword, AFXStringKeyword, AFXSymConstKeyword, AFXTogglableKeyword,
        AFXTupleKeyword, and AFXTableKeyword.
        """

    def getValueAsString(self):
        """Returns the text string that represents the current keyword value.

        Implemented in AFXBoolKeyword, AFXComSymConstKeyword, AFXComTableKeyword, AFXFloatKeyword,
        AFXIntKeyword, AFXObjectKeyword, AFXStringKeyword, AFXSymConstKeyword, AFXTogglableKeyword, and
        AFXTupleKeyword.
        """

    def isActive(self):
        """Returns True if the keyword is active."""

    def isRequired(self):
        """Returns True if the keyword is a required argument of the host command; or returns False if the
        keyword is optional."""

    def isValueChanged(self):
        """Returns True if the keyword value differs from its previous value.

        Implemented in AFXBoolKeyword, AFXComSymConstKeyword, AFXComTableKeyword, AFXFloatKeyword,
        AFXIntKeyword, AFXObjectKeyword, AFXStringKeyword, AFXTogglableKeyword, and AFXTupleKeyword.
        """

    def setRequired(self, val: bool):
        """Sets this object as a required keyword of the host command.

        Parameters
        ----------
        val : bool
        """

    def setSetupCommands(self, cmds: str):
        """Sets variable initialization commands needed by the keyword.

        Parameters
        ----------
        cmds : str
        """

    def setValueToDefault(self, ignoreUnspecified: bool = False):
        """Sets the keyword value to its default.

        Parameters
        ----------
        ignoreUnspecified : bool
            Ignore setting the value if the default is unspecified.
        """

    def setValueToPrevious(self):
        """Sets the keyword value to its previous value.

        Implemented in AFXBoolKeyword, AFXComSymConstKeyword, AFXComTableKeyword, AFXFloatKeyword,
        AFXIntKeyword, AFXObjectKeyword, AFXStringKeyword, AFXTogglableKeyword, and AFXTupleKeyword.
        """

    def syncPreviousValue(self):
        """Sets the keyword's previous value to its current value.

        Implemented in AFXBoolKeyword, AFXComSymConstKeyword, AFXComTableKeyword, AFXFloatKeyword,
        AFXIntKeyword, AFXObjectKeyword, AFXStringKeyword, AFXTogglableKeyword, and AFXTupleKeyword.
        """
